<?php
namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;

class RNFFactory {
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function findFiliais($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            SELECT DISTINCT F.CDFILIAL, NMFILIAL, F.CDFILIALEXT
                FROM FILIAL F
                    JOIN FILIOPER FO
                        ON F.CDFILIAL = FO.CDFILIAL
                WHERE FO.CDOPERADOR = :P_CDOPERADOR
                    AND ((F.CDFILIAL IN (:CDFILIAL)) 
                        OR (:TCDFILIAL = 'T')
                    )
                    AND ((F.CDFILIAL IN (:CDFILIAL)) 
                        OR (:TCDFILIAL = 'T')
                    )
                    AND EXISTS (
                        SELECT 1
                            FROM HEADERINTSAPFOS H
                            WHERE H.COD_UNIDADE_NL = F.CDFILIALEXT
                    )
                    AND ISNUMBER(F.CDFILIALEXT) = 'S'
                    AND EXISTS (
                        SELECT 1
                            FROM HEADERINTSAPFOS C
                            WHERE C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
                                AND C.COD_UNIDADE_NL = F.CDFILIALEXT
                                AND ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                    )
            ORDER BY 1
        ";

        $queryPostgre = "
            SELECT DISTINCT F.CDFILIAL AS 'CDFILIAL', NMFILIAL AS 'NMFILIAL', F.CDFILIALEXT AS 'CDFILIALEXT'
                FROM FILIAL F
                    JOIN FILIOPER FO
                        ON F.CDFILIAL = FO.CDFILIAL
                WHERE FO.CDOPERADOR = :P_CDOPERADOR
                    AND ((F.CDFILIAL IN (:CDFILIAL)) 
                        OR (:TCDFILIAL = 'T')
                    )
                    AND EXISTS (
                        SELECT 1
                            FROM HEADERINTSAPFOS H
                            WHERE H.COD_UNIDADE_NL = F.CDFILIALEXT
                    )
                    AND ISNUMERIC(F.CDFILIALEXT) = 'S'
                    AND EXISTS (
                        SELECT 1
                            FROM HEADERINTSAPFOS C
                            WHERE C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
                                AND C.COD_UNIDADE_NL = F.CDFILIALEXT
                                AND ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                    )
            ORDER BY 1
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    public function findCabecalho($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            SELECT NUM_SEQ_HEADER, COD_EMP_SAP, COD_UNIDADE_SAP, COD_UNIDADE_NL || ' - ' || F.NMFILIAL AS LABELFILIAL, 
                    NUM_RNF, DTA_RNF, HRA_RNF, DES_ARQUIVO, QTD_NOTAS, TIP_LOTE,
                    DTA_IMPORTACAO, DTA_PROCESS, HRA_PROCESS, IND_EXCLUSAO, DTH_GERACAO_ARQ_CLI,
                    SEQ_ESCOLA_BRITANICA, QTD_NOTAS_ATIVAS, NUM_HEADER_ORI, DTA_REPROCESS, HRA_REPROCESS,
                    DECODE(DTA_IMPORTACAO, NULL, 'P', DECODE(DTA_PROCESS, NULL, 'I', 'R')) AS IDSTATUS
                FROM HEADERINTSAPFOS C
                    JOIN (
                        SELECT TO_NUMBER(CDFILIALEXT) AS CDFILIALEXT, NMFILIAL
                            FROM FILIAL F
                                JOIN FILIOPER FO
                                    ON F.CDFILIAL = FO.CDFILIAL
                            WHERE FO.CDOPERADOR = :P_CDOPERADOR
                                AND ((F.CDFILIAL IN (:CDFILIAL)) 
                                    OR (:TCDFILIAL = 'T')
                                )
                                AND ISNUMBER(F.CDFILIALEXT) = 'S'
                    ) F
                        ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                    AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ORDER BY 1
        ";

        $queryPostgre = "
            SELECT NUM_SEQ_HEADER AS 'NUM_SEQ_HEADER', COD_EMP_SAP AS 'COD_EMP_SAP', COD_UNIDADE_SAP AS 'COD_UNIDADE_SAP', 
                    COD_UNIDADE_NL || ' - ' || F.NMFILIAL AS 'LABELFILIAL', NUM_RNF AS 'NUM_RNF', DTA_RNF AS 'DTA_RNF',
                    HRA_RNF AS 'HRA_RNF', DES_ARQUIVO AS 'DES_ARQUIVO', QTD_NOTAS AS 'QTD_NOTAS', TIP_LOTE AS 'TIP_LOTE',
                    DTA_IMPORTACAO AS 'DTA_IMPORTACAO', DTA_PROCESS AS 'DTA_PROCESS', HRA_PROCESS AS 'HRA_PROCESS', 
                    IND_EXCLUSAO AS 'IND_EXCLUSAO', DTH_GERACAO_ARQ_CLI AS 'DTH_GERACAO_ARQ_CLI',
                    SEQ_ESCOLA_BRITANICA AS 'SEQ_ESCOLA_BRITANICA', QTD_NOTAS_ATIVAS AS 'QTD_NOTAS_ATIVAS', 
                    NUM_HEADER_ORI AS 'NUM_HEADER_ORI', DTA_REPROCESS AS 'DTA_REPROCESS', HRA_REPROCESS AS 'HRA_REPROCESS',
                    CASE
                        WHEN DTA_IMPORTACAO IS NULL THEN 'P'
                        WHEN DTA_PROCESS IS NULL THEN 'I'
                        ELSE 'R'
                    END AS 'IDSTATUS'
                FROM HEADERINTSAPFOS C
                    JOIN (
                        SELECT TO_NUMBER(CDFILIALEXT, '9999') AS CDFILIALEXT, NMFILIAL
                            FROM FILIAL F
                                JOIN FILIOPER FO
                                    ON F.CDFILIAL = FO.CDFILIAL
                            WHERE FO.CDOPERADOR = :P_CDOPERADOR
                                AND ((F.CDFILIAL IN (:CDFILIAL)) 
                                    OR (:TCDFILIAL = 'T')
                                )
                                AND ISNUMERIC(F.CDFILIALEXT) = 'S'
                    ) F
                        ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                    AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ORDER BY 1
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    public function findNotas($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            WITH F AS (
                    SELECT CDFILIALEXT
                        FROM FILIAL F
                            JOIN FILIOPER FO
                                ON F.CDFILIAL = FO.CDFILIAL
                        WHERE FO.CDOPERADOR = :P_CDOPERADOR
                            AND ((F.CDFILIAL IN (:CDFILIAL)) 
                                OR (:TCDFILIAL = 'T')
                            )
                            AND ISNUMBER(F.CDFILIALEXT) = 'S'
                ),
                C AS (
                    SELECT NUM_SEQ_HEADER, COD_EMP_SAP, COD_UNIDADE_SAP, NUM_RNF, DTA_RNF,
                            HRA_RNF, DES_ARQUIVO, QTD_NOTAS, TIP_LOTE, COD_UNIDADE_NL,
                            DTA_IMPORTACAO, DTA_PROCESS, HRA_PROCESS, IND_EXCLUSAO, DTH_GERACAO_ARQ_CLI,
                            SEQ_ESCOLA_BRITANICA, QTD_NOTAS_ATIVAS, NUM_HEADER_ORI, DTA_REPROCESS, HRA_REPROCESS
                        FROM HEADERINTSAPFOS C
                            JOIN F
                                ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                        WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                            AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
                ORDER BY 1
                )
            SELECT BODY.NUM_SEQ_HEADER, BODY.NUM_SEQ_CORPO, BODY.NUM_CNPJ_CPF, BODY.COD_SERIE, BODY.NUM_NOTA,
                    BODY.VLR_TOTAL, BODY.DTA_EMISSAO, BODY.NUM_DOC_CONT, BODY.NUM_DOC_FISC
                FROM BODYINTSAPFOS BODY
                    JOIN C
                        ON BODY.NUM_SEQ_HEADER = C.NUM_SEQ_HEADER
            ORDER BY 1
        ";

        $queryPostgre = "
            WITH F AS (
                SELECT CDFILIALEXT
                    FROM FILIAL F
                        JOIN FILIOPER FO
                            ON F.CDFILIAL = FO.CDFILIAL
                    WHERE FO.CDOPERADOR = :P_CDOPERADOR
                        AND ((F.CDFILIAL IN (:CDFILIAL)) 
                            OR (:TCDFILIAL = 'T')
                        )
                        AND ISNUMERIC(F.CDFILIALEXT) = 'S'
            ),
            C AS (
                SELECT NUM_SEQ_HEADER, COD_EMP_SAP, COD_UNIDADE_SAP, NUM_RNF, DTA_RNF,
                        HRA_RNF, DES_ARQUIVO, QTD_NOTAS, TIP_LOTE, COD_UNIDADE_NL,
                        DTA_IMPORTACAO, DTA_PROCESS, HRA_PROCESS, IND_EXCLUSAO, DTH_GERACAO_ARQ_CLI,
                        SEQ_ESCOLA_BRITANICA, QTD_NOTAS_ATIVAS, NUM_HEADER_ORI, DTA_REPROCESS, HRA_REPROCESS
                    FROM HEADERINTSAPFOS C
                        JOIN F
                            ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                    WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                        AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ORDER BY 1
            )
            SELECT BODY.NUM_SEQ_HEADER AS 'NUM_SEQ_HEADER', BODY.NUM_SEQ_CORPO AS 'NUM_SEQ_CORPO', 
                    BODY.NUM_CNPJ_CPF AS 'NUM_CNPJ_CPF', BODY.COD_SERIE AS 'COD_SERIE', 
                    BODY.NUM_NOTA AS 'NUM_NOTA', BODY.VLR_TOTAL AS 'VLR_TOTAL', 
                    BODY.DTA_EMISSAO AS 'DTA_EMISSAO', BODY.NUM_DOC_CONT AS 'NUM_DOC_CONT', 
                    BODY.NUM_DOC_FISC AS 'NUM_DOC_FISC'
                FROM BODYINTSAPFOS BODY
                    JOIN C
                        ON BODY.NUM_SEQ_HEADER = C.NUM_SEQ_HEADER
            ORDER BY 1
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    public function findItens($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            WITH F AS (
                SELECT CDFILIALEXT
                    FROM FILIAL F
                        JOIN FILIOPER FO
                            ON F.CDFILIAL = FO.CDFILIAL
                    WHERE FO.CDOPERADOR = :P_CDOPERADOR
                        AND ((F.CDFILIAL IN (:CDFILIAL)) 
                            OR (:TCDFILIAL = 'T')
                        )
                        AND ISNUMBER(F.CDFILIALEXT) = 'S'
            ),
            C AS (
                SELECT C.NUM_SEQ_HEADER
                    FROM HEADERINTSAPFOS C
                        JOIN F
                            ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                    WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                        AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ),
            BODY AS (
                SELECT BODY.NUM_SEQ_HEADER, BODY.NUM_SEQ_CORPO
                    FROM BODYINTSAPFOS BODY
                        JOIN C
                            ON BODY.NUM_SEQ_HEADER = C.NUM_SEQ_HEADER
            )
            SELECT I.NUM_SEQ_HEADER, I.NUM_SEQ_CORPO, I.NUM_SEQ_ITEM, I.NUM_ITEM, 
                    I.TIP_DOCUMENTO, I.VLR_ITEM, I.NUM_CFOP, I.VLR_DESCONTO,
                    I.VLR_FRETE, I.VLR_SEGURO, I.VLR_OUTRAS_DESP, I.COD_ITEM, 
                    I.QTD_ITEM, I.VLR_UNITARIO, I.NUM_NOTA_ORIGEM, I.COD_SERIE_ORIGEM, 
                    I.DES_ITEM, I.COD_CC, I.COD_NAT_R_PISCOF, I.COD_TP_CR_PISCOF, 
                    I.COD_NT_BC_PISCOF, I.COD_NAT_FRETE,
                    SUBSTR(I.COD_NBM, 1, 4) || '.' || SUBSTR(I.COD_NBM, 5, 2) || '.' || SUBSTR(I.COD_NBM, 7, 2) AS COD_NBM,
                    I.COD_ITEM_NL,
                    TRUNC((NVL(I.QTD_ITEM, 1) * NVL(I.VLR_UNITARIO, 0)) - NVL(I.VLR_DESCONTO, 0) - NVL(I.VLR_FRETE, 0) - NVL(I.VLR_SEGURO, 0) + NVL(I.VLR_OUTRAS_DESP, 0), 2) AS VRTOTAL
                FROM ITEMINTSAPFOS I
                    JOIN BODY
                        ON I.NUM_SEQ_HEADER = BODY.NUM_SEQ_HEADER
                            AND I.NUM_SEQ_CORPO = BODY.NUM_SEQ_CORPO
        ";

        $queryPostgre = "
            WITH F AS (
                SELECT CDFILIALEXT
                    FROM FILIAL F
                        JOIN FILIOPER FO
                            ON F.CDFILIAL = FO.CDFILIAL
                    WHERE FO.CDOPERADOR = :P_CDOPERADOR
                        AND ((F.CDFILIAL IN (:CDFILIAL)) 
                            OR (:TCDFILIAL = 'T')
                        )
                        AND ISNUMERIC(F.CDFILIALEXT) = 'S'
            ),
            C AS (
                SELECT C.NUM_SEQ_HEADER
                    FROM HEADERINTSAPFOS C
                        JOIN F
                            ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                    WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                        AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ),
            BODY AS (
                SELECT BODY.NUM_SEQ_HEADER, BODY.NUM_SEQ_CORPO
                    FROM BODYINTSAPFOS BODY
                        JOIN C
                            ON BODY.NUM_SEQ_HEADER = C.NUM_SEQ_HEADER
            )
            SELECT I.NUM_SEQ_HEADER AS 'NUM_SEQ_HEADER', I.NUM_SEQ_CORPO AS 'NUM_SEQ_CORPO', 
                    I.NUM_SEQ_ITEM AS 'NUM_SEQ_ITEM', I.NUM_ITEM AS 'NUM_ITEM', 
                    I.TIP_DOCUMENTO AS 'TIP_DOCUMENTO', I.VLR_ITEM AS 'VLR_ITEM', 
                    I.NUM_CFOP AS 'NUM_CFOP', I.VLR_DESCONTO AS 'VLR_DESCONTO',
                    I.VLR_FRETE AS 'VLR_FRETE', I.VLR_SEGURO AS 'VLR_SEGURO', 
                    I.VLR_OUTRAS_DESP AS 'VLR_OUTRAS_DESP', I.COD_ITEM AS 'COD_ITEM', 
                    I.QTD_ITEM AS 'QTD_ITEM', I.VLR_UNITARIO AS 'VLR_UNITARIO', 
                    I.NUM_NOTA_ORIGEM AS 'NUM_NOTA_ORIGEM', I.COD_SERIE_ORIGEM AS 'COD_SERIE_ORIGEM', 
                    I.DES_ITEM AS 'DES_ITEM', I.COD_CC AS 'COD_CC', I.COD_NAT_R_PISCOF AS 'COD_NAT_R_PISCOF', 
                    I.COD_TP_CR_PISCOF AS 'COD_TP_CR_PISCOF', I.COD_NT_BC_PISCOF AS 'COD_NT_BC_PISCOF', I.COD_NAT_FRETE AS 'COD_NAT_FRETE',
                    SUBSTR(I.COD_NBM, 1, 4) || '.' || SUBSTR(I.COD_NBM, 5, 2) || '.' || SUBSTR(I.COD_NBM, 7, 2) AS 'COD_NBM',
                    I.COD_ITEM_NL AS 'COD_ITEM_NL',
                    TRUNC((COALESCE(I.QTD_ITEM, 1) * COALESCE(I.VLR_UNITARIO, 0)) - COALESCE(I.VLR_DESCONTO, 0) - COALESCE(I.VLR_FRETE, 0) - COALESCE(I.VLR_SEGURO, 0) + COALESCE(I.VLR_OUTRAS_DESP, 0), 2) AS 'VRTOTAL'
                FROM ITEMINTSAPFOS I
                    JOIN BODY
                        ON I.NUM_SEQ_HEADER = BODY.NUM_SEQ_HEADER
                            AND I.NUM_SEQ_CORPO = BODY.NUM_SEQ_CORPO
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    public function findImpostos($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            WITH F AS (
                SELECT CDFILIALEXT
                    FROM FILIAL F
                        JOIN FILIOPER FO
                            ON F.CDFILIAL = FO.CDFILIAL
                    WHERE FO.CDOPERADOR = :P_CDOPERADOR
                        AND ((F.CDFILIAL IN (:CDFILIAL)) 
                            OR (:TCDFILIAL = 'T')
                        )
                        AND ISNUMBER(F.CDFILIALEXT) = 'S'
            ), 
            C AS (
                SELECT C.NUM_SEQ_HEADER
                    FROM HEADERINTSAPFOS C
                        JOIN F
                            ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                    WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                        AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ),
            BODY AS (
                SELECT BODY.NUM_SEQ_HEADER, BODY.NUM_SEQ_CORPO
                    FROM BODYINTSAPFOS BODY
                        JOIN C
                            ON BODY.NUM_SEQ_HEADER = C.NUM_SEQ_HEADER
            ),
            ITEM AS (
                SELECT I.NUM_SEQ_HEADER, I.NUM_SEQ_CORPO, I.NUM_SEQ_ITEM
                    FROM ITEMINTSAPFOS I
                        JOIN BODY
                            ON I.NUM_SEQ_HEADER = BODY.NUM_SEQ_HEADER
                                AND I.NUM_SEQ_CORPO = BODY.NUM_SEQ_CORPO
            )
            SELECT IMP.NUM_SEQ_HEADER, IMP.NUM_SEQ_CORPO, IMP.NUM_SEQ_ITEM, IMP.NUM_SEQ_IMPOSTO, 
                    IMP.TIP_IMPOSTO, IMP.VLR_BASE, IMP.VLR_RED_BASE, IMP.VLR_OUTRAS, IMP.PER_IMPOSTO, 
                    IMP.VLR_IMPOSTO, IMP.TIP_TRIBUTACAO
                FROM IMPOSTOINTSAPFOS IMP
                    JOIN ITEM
                        ON ITEM.NUM_SEQ_HEADER = IMP.NUM_SEQ_HEADER
                            AND ITEM.NUM_SEQ_CORPO = IMP.NUM_SEQ_CORPO
                            AND ITEM.NUM_SEQ_ITEM = IMP.NUM_SEQ_ITEM
            ORDER BY 1, 2, 3, 4
        ";

        $queryPostgre = "
            WITH F AS (
                SELECT CDFILIALEXT
                    FROM FILIAL F
                        JOIN FILIOPER FO
                            ON F.CDFILIAL = FO.CDFILIAL
                    WHERE FO.CDOPERADOR = :P_CDOPERADOR
                        AND ((F.CDFILIAL IN (:CDFILIAL)) 
                            OR (:TCDFILIAL = 'T')
                        )
                        AND ISNUMERIC(F.CDFILIALEXT) = 'S'
            ), 
            C AS (
                SELECT C.NUM_SEQ_HEADER
                    FROM HEADERINTSAPFOS C
                        JOIN F
                            ON C.COD_UNIDADE_NL = F.CDFILIALEXT
                    WHERE ((C.NUM_RNF BETWEEN :P_NUMINI AND :P_NUMFIN) OR (:P_NUMINI IS NULL AND :P_NUMFIN IS NULL))
                        AND C.DTA_RNF BETWEEN :P_DTINI AND :P_DTFIN
            ),
            BODY AS (
                SELECT BODY.NUM_SEQ_HEADER, BODY.NUM_SEQ_CORPO
                    FROM BODYINTSAPFOS BODY
                        JOIN C
                            ON BODY.NUM_SEQ_HEADER = C.NUM_SEQ_HEADER
            ),
            ITEM AS (
                SELECT I.NUM_SEQ_HEADER, I.NUM_SEQ_CORPO, I.NUM_SEQ_ITEM
                    FROM ITEMINTSAPFOS I
                        JOIN BODY
                            ON I.NUM_SEQ_HEADER = BODY.NUM_SEQ_HEADER
                                AND I.NUM_SEQ_CORPO = BODY.NUM_SEQ_CORPO
            )
            SELECT IMP.NUM_SEQ_HEADER AS 'NUM_SEQ_HEADER', IMP.NUM_SEQ_CORPO AS 'NUM_SEQ_CORPO', IMP.NUM_SEQ_ITEM AS 'NUM_SEQ_ITEM', 
                    IMP.NUM_SEQ_IMPOSTO AS 'NUM_SEQ_IMPOSTO', IMP.TIP_IMPOSTO AS 'TIP_IMPOSTO', IMP.VLR_BASE AS 'VLR_BASE', 
                    IMP.VLR_RED_BASE AS 'VLR_RED_BASE', IMP.VLR_OUTRAS AS 'VLR_OUTRAS', IMP.PER_IMPOSTO AS 'PER_IMPOSTO', 
                    IMP.VLR_IMPOSTO AS 'VLR_IMPOSTO', IMP.TIP_TRIBUTACAO AS 'TIP_TRIBUTACAO'
                FROM IMPOSTOINTSAPFOS IMP
                    JOIN ITEM
                        ON ITEM.NUM_SEQ_HEADER = IMP.NUM_SEQ_HEADER
                            AND ITEM.NUM_SEQ_CORPO = IMP.NUM_SEQ_CORPO
                            AND ITEM.NUM_SEQ_ITEM = IMP.NUM_SEQ_ITEM
            ORDER BY 1, 2, 3, 4
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    public function gerarRNF($params) {
        $this->procedureValidacoesRNF($params);
            
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            SELECT DSVALIDACAO
                FROM VALIDACAORNF
            ORDER BY 1
        ";

        $queryPostgre = "
            SELECT DSVALIDACAO
                FROM VALIDACAORNF
            ORDER BY 1
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        $result = $connection->fetchAll($queryOracle, $params);

        if(!empty($result)) {
            throw new \Exception("Foram encontradas inconsistências na validação da geração. Deseja verificá-las? As RNF não serão geradas enquanto as inconsistências não forem corrigidas.", 1);
        } else {
            // valida se existe vendas duplicadas
            $vendasDuplicadas = $this->validaVendasDuplicadas($params);

            if(!empty($vendasDuplicadas)) {
                throw new \Exception("Operação bloqueada. Existem vendas duplicadas no período informado. Verifique o relatório Comparativo de Vendas MSDE x Vendas ORACLE e corrija as diferenças para prosseguir com a geração de RNF.", 2);
            } else {
                // roda procedure GERARNFSAP
                $this->procedureGeraRNFSAP($params);

                return true;
            }
        }
    }

    private function procedureValidacoesRNF($params) {
        try {
            // executando a procedure
            $this->connection->beginTransaction();
            
            // cria a procedure
            $procedure = new StoredProcedure($this->connection, 'VALIDACOES_RNF');
            
            // coloca os parametros
            $procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $params['CDFILIAL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $params['CDOPERADOR'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $params['DATAINI'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $params['DATAFIN'],    Param::PARAM_TYPE_STR));
            
            // executa a procedure
            $procedure->execute();
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    private function validaVendasDuplicadas($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            SELECT NVL(V.NRNOTAFISCALCE, V.NRCUPOMFIS) AS CUPOM, V.DTVENDA, V.NRSEQVENDA, V.CDFILIAL, V.CDCAIXA
                FROM VENDA V
                WHERE ((V.CDFILIAL IN (:P_CDFILIAL)) OR (:TCDFILIAL = 'T'))
                    AND V.DTENTRVENDA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                    AND (
                        EXISTS (
                            SELECT NRNOTAFISCALCE
                                FROM VENDA
                                WHERE CDFILIAL         = V.CDFILIAL
                                    AND CDCAIXA        = V.CDCAIXA
                                    AND DTENTRVENDA    = V.DTENTRVENDA
                                    AND NRNOTAFISCALCE = V.NRNOTAFISCALCE
                                    AND NRSEQVENDA    <> V.NRSEQVENDA
                        ) OR EXISTS (
                            SELECT DTVENDA
                                FROM VENDA
                                WHERE CDFILIAL      = V.CDFILIAL
                                    AND CDCAIXA     = V.CDCAIXA
                                    AND DTENTRVENDA = V.DTENTRVENDA
                                    AND DTVENDA     = V.DTVENDA
                                    AND NRSEQVENDA <> V.NRSEQVENDA
                        )
                    )

            MINUS
                
            SELECT NVL(V.NRNOTAFISCALCE, V.NRCUPOMFIS) AS CUPOM, V.DTVENDA, MAX(V.NRSEQVENDA) AS NRSEQVENDA, V.CDFILIAL, V.CDCAIXA
                FROM VENDA V
                WHERE ((V.CDFILIAL IN (:P_CDFILIAL)) OR (:TCDFILIAL = 'T'))
                    AND V.DTENTRVENDA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                    AND (
                        EXISTS (
                            SELECT NRNOTAFISCALCE
                                FROM VENDA
                                WHERE CDFILIAL         = V.CDFILIAL
                                    AND CDCAIXA        = V.CDCAIXA
                                    AND DTENTRVENDA    = V.DTENTRVENDA
                                    AND NRNOTAFISCALCE = V.NRNOTAFISCALCE
                                    AND NRSEQVENDA    <> V.NRSEQVENDA
                        ) OR EXISTS (
                            SELECT DTVENDA
                                FROM VENDA
                                WHERE CDFILIAL      = V.CDFILIAL
                                    AND CDCAIXA     = V.CDCAIXA
                                    AND DTENTRVENDA = V.DTENTRVENDA
                                    AND DTVENDA     = V.DTVENDA
                                    AND NRSEQVENDA <> V.NRSEQVENDA
                        )
                    )
                GROUP BY NVL(V.NRNOTAFISCALCE, V.NRCUPOMFIS), V.DTVENDA, V.CDFILIAL, V.CDCAIXA
        ";

        $queryPostgre = "
            SELECT COALESCE(V.NRNOTAFISCALCE, V.NRCUPOMFIS) AS CUPOM, V.DTVENDA AS 'DTVENDA', V.NRSEQVENDA AS 'NRSEQVENDA', 
                    V.CDFILIAL AS 'CDFILIAL', V.CDCAIXA AS 'CDCAIXA'
                FROM VENDA V
                WHERE ((V.CDFILIAL IN (:P_CDFILIAL)) OR (:TCDFILIAL = 'T'))
                    AND V.DTENTRVENDA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                    AND (
                        EXISTS (
                            SELECT NRNOTAFISCALCE
                                FROM VENDA
                                WHERE CDFILIAL         = V.CDFILIAL
                                    AND CDCAIXA        = V.CDCAIXA
                                    AND DTENTRVENDA    = V.DTENTRVENDA
                                    AND NRNOTAFISCALCE = V.NRNOTAFISCALCE
                                    AND NRSEQVENDA    <> V.NRSEQVENDA
                        ) OR EXISTS (
                            SELECT DTVENDA
                                FROM VENDA
                                WHERE CDFILIAL      = V.CDFILIAL
                                    AND CDCAIXA     = V.CDCAIXA
                                    AND DTENTRVENDA = V.DTENTRVENDA
                                    AND DTVENDA     = V.DTVENDA
                                    AND NRSEQVENDA <> V.NRSEQVENDA
                        )
                    )

            MINUS
                
            SELECT COALESCE(V.NRNOTAFISCALCE, V.NRCUPOMFIS) AS 'CUPOM', V.DTVENDA AS 'DTVENDA', MAX(V.NRSEQVENDA) AS 'NRSEQVENDA', 
                    V.CDFILIAL AS 'CDFILIAL', V.CDCAIXA AS 'CDCAIXA'
                FROM VENDA V
                WHERE ((V.CDFILIAL IN (:P_CDFILIAL)) OR (:TCDFILIAL = 'T'))
                    AND V.DTENTRVENDA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                    AND (
                        EXISTS (
                            SELECT NRNOTAFISCALCE
                                FROM VENDA
                                WHERE CDFILIAL         = V.CDFILIAL
                                    AND CDCAIXA        = V.CDCAIXA
                                    AND DTENTRVENDA    = V.DTENTRVENDA
                                    AND NRNOTAFISCALCE = V.NRNOTAFISCALCE
                                    AND NRSEQVENDA    <> V.NRSEQVENDA
                        ) OR EXISTS (
                            SELECT DTVENDA
                                FROM VENDA
                                WHERE CDFILIAL      = V.CDFILIAL
                                    AND CDCAIXA     = V.CDCAIXA
                                    AND DTENTRVENDA = V.DTENTRVENDA
                                    AND DTVENDA     = V.DTVENDA
                                    AND NRSEQVENDA <> V.NRSEQVENDA
                        )
                    )
                GROUP BY COALESCE(V.NRNOTAFISCALCE, V.NRCUPOMFIS), V.DTVENDA, V.CDFILIAL, V.CDCAIXA
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    private function procedureGeraRNFSAP($params) {
        try {
            // executando a procedure
            $this->connection->beginTransaction();
                
            // cria a procedure
            $procedure = new StoredProcedure($this->connection, 'GERARNFSAP');
            
            // coloca os parametros
            $procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $params['CDFILIAL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $params['CDOPERADOR'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $params['DATAINI'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $params['DATAFIN'],    Param::PARAM_TYPE_STR));
            
            // executa a procedure
            $procedure->execute();
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function validaGeracaoRNF($params) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            SELECT DISTINCT 1
                FROM (
                    SELECT F.CDFILIAL, F.CDLOJA, F.CDCAIXA, FILIALSAPRNF(F.CDFILIAL) AS CDFILISAP
                        FROM V_FILIALLOJACAIXA F
                            JOIN FILIOPER FO
                                ON F.CDFILIAL = FO.CDFILIAL
                        WHERE ((F.CDFILIAL IN (:P_CDFILIAL)) OR (:TCDFILIAL = 'T'))
                        AND FO.CDOPERADOR = :P_CDOPERADOR
                ) FO
                    JOIN HEADERINTSAPFOS H
                        ON H.COD_UNIDADE_SAP = FO.CDFILISAP
                WHERE H.DTA_RNF BETWEEN :P_DTINICIAL AND :P_DTFINAL
        ";

        $queryPostgre = "
            SELECT DISTINCT 1
                FROM (
                    SELECT F.CDFILIAL, F.CDLOJA, F.CDCAIXA, FILIALSAPRNF(F.CDFILIAL) AS 'CDFILISAP'
                        FROM V_FILIALLOJACAIXA F
                            JOIN FILIOPER FO
                                ON F.CDFILIAL = FO.CDFILIAL
                        WHERE ((F.CDFILIAL IN (:P_CDFILIAL)) OR (:TCDFILIAL = 'T'))
                        AND FO.CDOPERADOR = :P_CDOPERADOR
                ) FO
                    JOIN HEADERINTSAPFOS H
                        ON H.COD_UNIDADE_SAP = FO.CDFILISAP
                WHERE H.DTA_RNF BETWEEN :P_DTINICIAL AND :P_DTFINAL
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }

    public function verificaIntegracao() {
        $params = [
            'P_OWNER' => $this->connection->getParams()['user']
        ];
        
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = "
            SELECT STATUS
                FROM DBA_SCHEDULER_JOB_RUN_DETAILS JOBS
                WHERE OWNER = UPPER(:P_OWNER)
                    AND JOB_NAME = 'JOB_AGENDARESTORNOSAP'
                    AND LOG_DATE = (
                        SELECT MAX(LOG_DATE)
                            FROM DBA_SCHEDULER_JOB_RUN_DETAILS MAXDT
                            WHERE MAXDT.JOB_NAME = JOBS.JOB_NAME
                                AND MAXDT.OWNER = JOBS.OWNER
                    )
        ";

        $queryPostgre = "
            SELECT STATUS AS 'STATUS'
                FROM DBA_SCHEDULER_JOB_RUN_DETAILS JOBS
                WHERE OWNER = UPPER(:P_OWNER)
                    AND JOB_NAME = 'JOB_AGENDARESTORNOSAP'
                    AND LOG_DATE = (
                        SELECT MAX(LOG_DATE)
                            FROM DBA_SCHEDULER_JOB_RUN_DETAILS MAXDT
                            WHERE MAXDT.JOB_NAME = JOBS.JOB_NAME
                                AND MAXDT.OWNER = JOBS.OWNER
                    )
        ";

        $arrayQueries = [
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        ];

        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);

        return $connection->fetchAll($queryOracle, $params);
    }
}